@@ -1,8 +1,9 @@
 import asyncio
 import functools
-from typing import Dict, List, Optional, Set, Union
+from typing import Dict, List, Optional, Set, Union, Type, Tuple
 
 from cosmpy.aerial.wallet import LocalWallet, PrivateKey
+from cosmpy.crypto.address import Address
 
 from uagents.asgi import ASGIServer
 from uagents.context import (
@@ -18,7 +19,7 @@ from uagents.models import Model, ErrorMessage
 from uagents.protocol import Protocol
 from uagents.resolver import Resolver, AlmanacResolver
 from uagents.storage import KeyValueStore, get_or_create_private_keys
-from uagents.network import get_ledger, get_reg_contract
+from uagents.network import get_ledger, get_reg_contract, wait_for_tx_to_complete
 from uagents.mailbox import MailboxClient
 from uagents.config import (
     REGISTRATION_FEE,
@@ -69,7 +70,6 @@ class Agent(Sink):
             if name is None:
                 self._wallet = LocalWallet.generate()
                 self._identity = Identity.generate()
-                self._name = self.address[0:16]
             else:
                 identity_key, wallet_key = get_or_create_private_keys(name)
                 self._wallet = LocalWallet(PrivateKey(wallet_key))
@@ -80,6 +80,8 @@ class Agent(Sink):
                 PrivateKey(derive_key_from_seed(seed, LEDGER_PREFIX, 0)),
                 prefix=LEDGER_PREFIX,
             )
+        if name is None:
+            self._name = self.address[0:16]
         self._logger = get_logger(self.name)
 
         # configure endpoints and mailbox
@@ -87,24 +89,24 @@ class Agent(Sink):
         self._use_mailbox = mailbox is not None
         if self._use_mailbox:
             self._mailbox = parse_mailbox_config(mailbox)
-            self._mailbox_client = MailboxClient(self, self._mailbox, self._logger)
+            self._mailbox_client = MailboxClient(self, self._logger)
             # if mailbox is provided, override endpoints with mailbox endpoint
             self._endpoints = [
-                {"url": f"{self._mailbox['base_url']}/v1/submit", "weight": 1}
+                {
+                    "url": f"{self.mailbox['http_prefix']}://{self.mailbox['base_url']}/v1/submit",
+                    "weight": 1,
+                }
             ]
-        if self._endpoints is None:
-            self._logger.warning(
-                "I have no endpoint and won't be able to receive external messages"
-            )
 
         self._ledger = get_ledger()
         self._reg_contract = get_reg_contract()
         self._storage = KeyValueStore(self.address[0:16])
+        self._interval_handlers: List[Tuple[IntervalCallback, float]] = []
         self._interval_messages: Set[str] = set()
         self._signed_message_handlers: Dict[str, MessageCallback] = {}
         self._unsigned_message_handlers: Dict[str, MessageCallback] = {}
-        self._models: Dict[str, Model] = {}
-        self._replies: Dict[str, Set[Model]] = {}
+        self._models: Dict[str, Type[Model]] = {}
+        self._replies: Dict[str, Set[Type[Model]]] = {}
         self._queries: Dict[str, asyncio.Future] = {}
         self._ctx = Context(
             self._identity.address,
@@ -129,7 +131,7 @@ class Agent(Sink):
         self._protocol = Protocol(name=self._name, version=self._version)
 
         # keep track of supported protocols
-        self.protocols = {}
+        self.protocols: Dict[str, Protocol] = {}
 
         # register with the dispatcher
         self._dispatcher.register(self.address, self)
@@ -139,9 +141,7 @@ class Agent(Sink):
 
     @property
     def name(self) -> str:
-        if self._name is not None:
-            return self._name
-        return self.address[:10]
+        return self._name
 
     @property
     def address(self) -> str:
@@ -151,6 +151,18 @@ class Agent(Sink):
     def wallet(self) -> LocalWallet:
         return self._wallet
 
+    @property
+    def storage(self) -> KeyValueStore:
+        return self._storage
+
+    @property
+    def mailbox(self) -> Dict[str, str]:
+        return self._mailbox
+
+    @mailbox.setter
+    def mailbox(self, config: Union[str, Dict[str, str]]):
+        self._mailbox = parse_mailbox_config(config)
+
     def sign(self, data: bytes) -> str:
         return self._identity.sign(data)
 
@@ -158,8 +170,9 @@ class Agent(Sink):
         return self._identity.sign_digest(digest)
 
     def sign_registration(self) -> str:
+        assert self._reg_contract.address is not None
         return self._identity.sign_registration(
-            self._reg_contract.address, self.get_registration_sequence()
+            str(self._reg_contract.address), self.get_registration_sequence()
         )
 
     def update_loop(self, loop):
@@ -168,9 +181,8 @@ class Agent(Sink):
     def update_queries(self, queries):
         self._queries = queries
 
-    async def register(self, ctx: Context):
-
-        agent_balance = ctx.ledger.query_bank_balance(ctx.wallet)
+    async def _register(self, ctx: Context):
+        agent_balance = ctx.ledger.query_bank_balance(Address(ctx.wallet.address()))
 
         if agent_balance < REGISTRATION_FEE:
             self._logger.warning(
@@ -185,7 +197,9 @@ class Agent(Sink):
             "register": {
                 "record": {
                     "service": {
-                        "protocols": list(self.protocols.values()),
+                        "protocols": list(
+                            map(lambda x: x.digest, self.protocols.values())
+                        ),
                         "endpoints": self._endpoints,
                     }
                 },
@@ -196,24 +210,19 @@ class Agent(Sink):
         }
 
         self._logger.info("Registering on Almanac contract...")
-        transaction = await self._loop.run_in_executor(
-            None,
-            functools.partial(
-                self._reg_contract.execute,
-                msg,
-                ctx.wallet,
-                funds=f"{REGISTRATION_FEE}{REGISTRATION_DENOM}",
-            ),
+        transaction = self._reg_contract.execute(
+            msg,
+            ctx.wallet,
+            funds=f"{REGISTRATION_FEE}{REGISTRATION_DENOM}",
         )
-        await self._loop.run_in_executor(None, transaction.wait_to_complete)
+        await wait_for_tx_to_complete(transaction.tx_hash)
         self._logger.info("Registering on Almanac contract...complete")
 
     def _schedule_registration(self):
-
         query_msg = {"query_records": {"agent_address": self.address}}
         response = self._reg_contract.query(query_msg)
 
-        if response["record"] == []:
+        if not response["record"]:
             contract_state = self._reg_contract.query({"query_contract_state": {}})
             expiry = contract_state.get("state").get("expiry_height")
             return expiry * BLOCK_INTERVAL
@@ -230,26 +239,28 @@ class Agent(Sink):
         return sequence
 
     def on_interval(
-        self, period: float, messages: Optional[Union[Model, Set[Model]]] = None
+        self,
+        period: float,
+        messages: Optional[Union[Type[Model], Set[Type[Model]]]] = None,
     ):
         return self._protocol.on_interval(period, messages)
 
     def on_query(
         self,
-        model: Model,
+        model: Type[Model],
         replies: Optional[Union[Model, Set[Model]]] = None,
     ):
         return self._protocol.on_query(model, replies)
 
     def on_message(
         self,
-        model: Model,
-        replies: Optional[Union[Model, Set[Model]]] = None,
+        model: Type[Model],
+        replies: Optional[Union[Type[Model], Set[Type[Model]]]] = None,
         allow_unverified: Optional[bool] = False,
     ):
         return self._protocol.on_message(model, replies, allow_unverified)
 
-    def on_event(self, event_type: str) -> EventCallback:
+    def on_event(self, event_type: str):
         def decorator_on_event(func: EventCallback) -> EventCallback:
             @functools.wraps(func)
             def handler(*args, **kwargs):
@@ -273,9 +284,7 @@ class Agent(Sink):
 
     def include(self, protocol: Protocol):
         for func, period in protocol.intervals:
-            task = self._loop.create_task(_run_interval(func, self._ctx, period))
-            self._background_tasks.add(task)
-            task.add_done_callback(self._background_tasks.discard)
+            self._interval_handlers.append((func, period))
 
         self._interval_messages.update(protocol.interval_messages)
 
@@ -301,22 +310,31 @@ class Agent(Sink):
                 self._replies[schema_digest] = protocol.replies[schema_digest]
 
         if protocol.digest is not None:
-            self.protocols[protocol.canonical_name] = protocol.digest
+            self.protocols[protocol.digest] = protocol
 
     async def handle_message(self, sender, schema_digest: str, message: JsonStr):
         await self._message_queue.put((schema_digest, sender, message))
 
-    async def startup(self):
+    async def _startup(self):
         for handler in self._on_startup:
             await handler(self._ctx)
 
-    async def shutdown(self):
+    async def _shutdown(self):
         for handler in self._on_shutdown:
             await handler(self._ctx)
 
     def setup(self):
         # register the internal agent protocol
         self.include(self._protocol)
+        self._loop.run_until_complete(self._startup())
+        self.start_background_tasks()
+
+    def start_background_tasks(self):
+        # Start the interval tasks
+        for func, period in self._interval_handlers:
+            task = self._loop.create_task(_run_interval(func, self._ctx, period))
+            self._background_tasks.add(task)
+            task.add_done_callback(self._background_tasks.discard)
 
         # start the background message queue processor
         task = self._loop.create_task(self._process_message_queue())
@@ -326,19 +344,23 @@ class Agent(Sink):
         # start the contract registration update loop
         if self._endpoints is not None:
             self._loop.create_task(
-                _run_interval(self.register, self._ctx, self._schedule_registration())
+                _run_interval(self._register, self._ctx, self._schedule_registration())
+            )
+        else:
+            self._logger.warning(
+                "I have no endpoint and won't be able to receive external messages"
             )
 
     def run(self):
         self.setup()
-        self._loop.run_until_complete(self.startup())
         try:
             if self._use_mailbox:
+                self._loop.create_task(self._mailbox_client.process_deletion_queue())
                 self._loop.run_until_complete(self._mailbox_client.run())
             else:
                 self._loop.run_until_complete(self._server.serve())
         finally:
-            self._loop.run_until_complete(self.shutdown())
+            self._loop.run_until_complete(self._shutdown())
 
     async def _process_message_queue(self):
         while True:
