@@ -1,12 +1,13 @@
 import hashlib
+from typing import Type, Union
 
 from pydantic import BaseModel
 
 
 class Model(BaseModel):
     @staticmethod
-    def build_schema_digest(model: "Model") -> str:
-        return (
+    def build_schema_digest(model: Union["Model", Type["Model"]]) -> str:
+        digest = (
             hashlib.sha256(
                 model.schema_json(indent=None, sort_keys=True).encode("utf8")
             )
@@ -14,6 +15,8 @@ class Model(BaseModel):
             .hex()
         )
 
+        return f"model:{digest}"
+
 
 class ErrorMessage(Model):
     error: str
